import ij.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;


public class Background_Extractor implements PlugInFilter {

	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		IJ.register(Background_Extractor.class);
		if (IJ.versionLessThan("1.32c"))
			return DONE;
		this.imp = imp;
		return DOES_32+SUPPORTS_MASKING;
	}

	public void run(ImageProcessor ip) {
		

		GenericDialog dia = new GenericDialog("Background Extractor", IJ.getInstance());
                String[] type = {"Mean", "Median", "Mode"};
                dia.addChoice("Normalize each image by:", type, "Mean");
                dia.addChoice("Combine stack by:", type, "Median");

		dia.showDialog();

		if (dia.wasCanceled()) return;

		

		int image_op = dia.getNextChoiceIndex();
                int stack_op = dia.getNextChoiceIndex();
                
                normalize(image_op, stack_op);

		imp.getProcessor().resetMinAndMax();
		imp.updateAndRepaintWindow();
	}
	



	void normalize(int image_op, int stack_op) {
		Analyzer analyzer = new Analyzer();
		ImageStack stack = imp.getStack();

		int size = stack.getSize();

		double measurement = 1.0;
		

		ImageProcessor ip = imp.getProcessor();



		
		for (int slice=1; slice<=size; slice++) {
			IJ.showStatus(" "+slice+"/"+size);
			
			//ip = stack.getProcessor(slice);
			
			imp.setSlice(slice);
			ImageStatistics stats = imp.getStatistics(Analyzer.getMeasurements());
			
			if(image_op==0) {
				//measurement = is.getStatistics(ip,1,null).mean;
				measurement = stats.mean;				
			}
			else if(image_op==1) {
				//measurement = is.getStatistics(ip,1,null).median;
				measurement = stats.median;
			}
			else if(image_op==2) {
				//measurement = is.getStatistics(ip,1,null).mode;
				measurement = stats.mode;
			}
			IJ.write("measurement = " +measurement);
			ip.multiply(1/measurement);

		}
		
		imp.unlock();
		
		if(stack_op==0) {
			IJ.run("Z Project...", "projection=Mean");
		}
		else if(stack_op==1) {
			IJ.run("Z Project...", "projection=Median");
		}
		else if(stack_op==2) {
			IJ.run("Z Project...", "projection=Mode");
		}

	}


}
