import ij.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;


public class Normalize_Values implements PlugInFilter {

	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		IJ.register(Background_Extractor.class);
		if (IJ.versionLessThan("1.32c"))
			return DONE;
		this.imp = imp;
		return DOES_ALL+SUPPORTS_MASKING;
	}

	public void run(ImageProcessor ip) {
		

		GenericDialog dia = new GenericDialog("Background Extractor", IJ.getInstance());
                String[] type = {"Mean", "Median", "Mode"};
                dia.addChoice("Normalize each image by:", type, "Mean");

		dia.showDialog();

		if (dia.wasCanceled()) return;

		

		int image_op = dia.getNextChoiceIndex();

                
		normalize(image_op);

		imp.getProcessor().resetMinAndMax();
		imp.updateAndRepaintWindow();
	}
	



	void normalize(int image_op) {
		Analyzer analyzer = new Analyzer();
		ImageStack stack = imp.getStack();

		int size = stack.getSize();

		double measurement = 1.0;
		double max = 0;
		double[] measurements = new double[size];

		ImageProcessor ip = imp.getProcessor();



		
		for (int slice=1; slice<=size; slice++) {
			IJ.showStatus(" "+slice+"/"+size);
			
			//ip = stack.getProcessor(slice);
			
			imp.setSlice(slice);
			ImageStatistics stats = imp.getStatistics(Analyzer.getMeasurements());
			
			if(image_op==0) {
				//measurement = is.getStatistics(ip,1,null).mean;
				measurement = stats.mean;				
			}
			else if(image_op==1) {
				//measurement = is.getStatistics(ip,1,null).median;
				measurement = stats.median;
			}
			else if(image_op==2) {
				//measurement = is.getStatistics(ip,1,null).mode;
				measurement = stats.mode;
			}
			
			if(measurement > max) {
				max = measurement;
			}
			
			measurements[slice-1] = measurement;

		}
		
		for (int slice=1; slice<=size; slice++) {
			IJ.showStatus(" "+slice+"/"+size);			
			imp.setSlice(slice);
			ip.multiply(max/measurements[slice-1]);
		}
		
		imp.unlock();


	}


}
